@@ -17,7 +17,6 @@ static const uint8_t signature[] = { 0xff, 0xd8 };
 struct jpg_error_manager {
     struct jpeg_error_mgr mgr;
     jmp_buf setjmp;
-    struct image* img;
 };
 
 static void jpg_error_exit(j_common_ptr jpg)
@@ -43,7 +42,6 @@ enum loader_status decode_jpeg(struct image* ctx, const uint8_t* data,
     }
 
     jpg.err = jpeg_std_error(&err.mgr);
-    err.img = ctx;
     err.mgr.error_exit = jpg_error_exit;
     if (setjmp(err.setjmp)) {
         image_free_frames(ctx);
