@@ -2,7 +2,7 @@
 // PNG format decoder.
 // Copyright (C) 2020 Artem Senichev <artemsen@gmail.com>
 
-#include "../loader.h"
+#include "png.h"
 
 #include <png.h>
 #include <setjmp.h>
@@ -16,6 +16,12 @@ struct mem_reader {
     size_t position;
 };
 
+// PNG memory writer
+struct mem_writer {
+    uint8_t** data;
+    size_t* size;
+};
+
 // PNG reader callback, see `png_rw_ptr` in png.h
 static void png_reader(png_structp png, png_bytep buffer, size_t size)
 {
@@ -28,6 +34,23 @@ static void png_reader(png_structp png, png_bytep buffer, size_t size)
     }
 }
 
+// PNG writer callback, see `png_rw_ptr` in png.h
+static void png_writer(png_structp png, png_bytep buffer, size_t size)
+{
+    struct mem_writer* writer = (struct mem_writer*)png_get_io_ptr(png);
+    const size_t old_size = *writer->size;
+    const size_t new_size = old_size + size;
+    uint8_t* new_ptr = realloc(*writer->data, new_size);
+
+    if (new_ptr) {
+        memcpy(new_ptr + old_size, buffer, size);
+        *writer->data = new_ptr;
+        *writer->size = new_size;
+    } else {
+        png_error(png, "No memory");
+    }
+}
+
 /**
  * Bind pixmap with PNG line-reading decoder.
  * @param pm pixmap to bind
@@ -35,7 +58,7 @@ static void png_reader(png_structp png, png_bytep buffer, size_t size)
  */
 static png_bytep* bind_pixmap(const struct pixmap* pm)
 {
-    png_bytep* ptr = malloc(pm->height * sizeof(png_bytep));
+    png_bytep* ptr = malloc(pm->height * sizeof(*ptr));
 
     if (ptr) {
         for (uint32_t i = 0; i < pm->height; ++i) {
@@ -308,6 +331,17 @@ enum loader_status decode_png(struct image* ctx, const uint8_t* data,
     rc = decode_single(ctx, png, info);
 #endif // PNG_APNG_SUPPORTED
 
+    // read text info
+    if (rc) {
+        png_text* txt;
+        int total;
+        if (png_get_text(png, info, &txt, &total)) {
+            for (int i = 0; i < total; ++i) {
+                image_add_meta(ctx, txt[i].key, "%s", txt[i].text);
+            }
+        }
+    }
+
     if (!rc) {
         image_free_frames(ctx);
     } else {
@@ -320,3 +354,73 @@ enum loader_status decode_png(struct image* ctx, const uint8_t* data,
 
     return rc ? ldr_success : ldr_fmterror;
 }
+
+bool encode_png(const struct image* ctx, uint8_t** data, size_t* size)
+{
+    png_struct* png = NULL;
+    png_info* info = NULL;
+    png_bytep* bind = NULL;
+    struct mem_writer writer = {
+        .data = data,
+        .size = size,
+    };
+
+    // create encoder
+    png = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+    if (!png) {
+        return false;
+    }
+    info = png_create_info_struct(png);
+    if (!info) {
+        png_destroy_write_struct(&png, NULL);
+        return false;
+    }
+
+    // setup error handling
+    if (setjmp(png_jmpbuf(png))) {
+        free(bind);
+        png_destroy_write_struct(&png, &info);
+        return false;
+    }
+
+    png_set_write_fn(png, &writer, png_writer, NULL);
+
+    // setup output: 8bit RGBA
+    png_set_IHDR(png, info, ctx->frames[0].pm.width, ctx->frames[0].pm.height,
+                 8, PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE,
+                 PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+    png_set_bgr(png);
+
+    // save meta info as text
+    if (ctx->info) {
+        const size_t info_sz = list_size(&ctx->info->list);
+        png_text* txt = calloc(1, info_sz * sizeof(png_text));
+        if (txt) {
+            size_t i = 0;
+            list_for_each(ctx->info, const struct image_info, it) {
+                txt[i].key = it->key;
+                txt[i].text = it->value;
+                ++i;
+            }
+            png_set_text(png, info, txt, info_sz);
+            free(txt);
+        }
+    }
+
+    png_write_info(png, info);
+
+    // encode image
+    bind = bind_pixmap(&ctx->frames[0].pm);
+    if (!bind) {
+        png_destroy_write_struct(&png, &info);
+        return false;
+    }
+    png_write_image(png, bind);
+    png_write_end(png, NULL);
+
+    // free resources
+    free(bind);
+    png_destroy_write_struct(&png, &info);
+
+    return true;
+}
