@@ -17,16 +17,6 @@
 #include <sys/timerfd.h>
 #include <unistd.h>
 
-// Configuration parameters
-#define CFG_CSECTION     "info"
-#define CFG_VSECTION     "info.viewer"
-#define CFG_GSECTION     "info.gallery"
-#define CFG_SHOW         "show"
-#define CFG_ITIMEOUT     "info_timeout"
-#define CFG_ITIMEOUT_DEF 5
-#define CFG_STIMEOUT     "status_timeout"
-#define CFG_STIMEOUT_DEF 3
-
 /** Display modes. */
 enum info_mode {
     mode_viewer,
@@ -34,8 +24,8 @@ enum info_mode {
     mode_off,
 };
 static const char* mode_names[] = {
-    [mode_viewer] = APP_MODE_VIEWER,
-    [mode_gallery] = APP_MODE_GALLERY,
+    [mode_viewer] = CFG_MODE_VIEWER,
+    [mode_gallery] = CFG_MODE_GALLERY,
     [mode_off] = "off",
 };
 #define MODES_NUM 2
@@ -44,6 +34,7 @@ static const char* mode_names[] = {
 /** Field names. */
 static const char* field_names[] = {
     [info_file_name] = "name",
+    [info_file_dir] = "dir",
     [info_file_path] = "path",
     [info_file_size] = "filesize",
     [info_image_format] = "format",
@@ -55,7 +46,6 @@ static const char* field_names[] = {
     [info_status] = "status",
 };
 #define FIELDS_NUM ARRAY_SIZE(field_names)
-// clang-format on
 
 /** Positions of text info block. */
 enum block_position {
@@ -67,29 +57,14 @@ enum block_position {
 };
 /** Block position names. */
 static const char* position_names[] = {
-    [pos_center] = "center",
-    [pos_top_left] = "top_left",
-    [pos_top_right] = "top_right",
-    [pos_bottom_left] = "bottom_left",
-    [pos_bottom_right] = "bottom_right",
+    [pos_center] = CFG_INFO_CN,
+    [pos_top_left] = CFG_INFO_TL,
+    [pos_top_right] = CFG_INFO_TR,
+    [pos_bottom_left] = CFG_INFO_BL,
+    [pos_bottom_right] = CFG_INFO_BR,
 };
 #define POSITION_NUM ARRAY_SIZE(position_names)
-
-// Default configuration
-static const char* default_viewer[] = {
-    [pos_center] = "none",
-    [pos_top_left] = "+name,+format,+filesize,+imagesize,+exif",
-    [pos_top_right] = "index",
-    [pos_bottom_left] = "scale,frame",
-    [pos_bottom_right] = "status",
-};
-static const char* default_gallery[] = {
-    [pos_center] = "none",
-    [pos_top_left] = "none",
-    [pos_top_right] = "none",
-    [pos_bottom_left] = "none",
-    [pos_bottom_right] = "name,status",
-};
+// clang-format on
 
 // Max number of lines in one positioned block
 #define MAX_LINES (FIELDS_NUM + 10 /* EXIF and duplicates */)
@@ -336,36 +311,36 @@ static void print_keyval(struct pixmap* wnd, enum block_position pos,
  */
 static void import_exif(const struct image* image)
 {
-    struct keyval* line;
-    const size_t buf_size = image->num_info * sizeof(*line);
+    struct keyval* lines;
+    const size_t num_entries = list_size(&image->info->list);
+    const size_t buf_size = num_entries * sizeof(*lines);
 
-    // free previuos lines
+    // free previous lines
     for (size_t i = 0; i < ctx.exif_num; ++i) {
         free(ctx.exif_lines[i].key.data);
         free(ctx.exif_lines[i].value.data);
     }
     ctx.exif_num = 0;
 
-    if (image->num_info == 0) {
+    if (num_entries == 0) {
         return;
     }
 
-    line = realloc(ctx.exif_lines, buf_size);
-    if (!line) {
+    lines = realloc(ctx.exif_lines, buf_size);
+    if (!lines) {
         return;
     }
-    memset(line, 0, buf_size);
+    memset(lines, 0, buf_size);
 
-    ctx.exif_num = image->num_info;
-    ctx.exif_lines = line;
+    ctx.exif_num = num_entries;
+    ctx.exif_lines = lines;
 
-    for (size_t i = 0; i < ctx.exif_num; ++i) {
-        const struct image_info* exif = &image->info[i];
+    list_for_each(image->info, const struct image_info, it) {
         char key[64];
-        snprintf(key, sizeof(key), "%s:", exif->key);
-        font_render(key, &line->key);
-        font_render(exif->value, &line->value);
-        ++line;
+        snprintf(key, sizeof(key), "%s:", it->key);
+        font_render(key, &lines->key);
+        font_render(it->value, &lines->value);
+        ++lines;
     }
 }
 
@@ -426,37 +401,34 @@ static bool parse_scheme(const char* config, struct block_scheme* scheme)
     return true;
 }
 
-void info_init(struct config* cfg)
+void info_init(const struct config* cfg)
 {
-    const char** defaults;
-    const char* section;
-    const char* position;
-    const char* format;
-
     for (size_t i = 0; i < MODES_NUM; ++i) {
-        defaults = (i == mode_viewer ? default_viewer : default_gallery);
-        section = (i == mode_viewer ? CFG_VSECTION : CFG_GSECTION);
+        const char* section;
+        section = (i == mode_viewer ? CFG_INFO_VIEWER : CFG_INFO_GALLERY);
         for (size_t j = 0; j < POSITION_NUM; ++j) {
-            position = position_names[j];
-            format = config_get_string(cfg, section, position, defaults[j]);
+            const char* position = position_names[j];
+            const char* format = config_get(cfg, section, position);
             if (!parse_scheme(format, &ctx.scheme[i][j])) {
                 config_error_val(section, format);
-                parse_scheme(defaults[j], &ctx.scheme[i][j]);
+                format = config_get_default(section, position);
+                parse_scheme(format, &ctx.scheme[i][j]);
             }
         }
     }
 
-    ctx.mode = config_get_bool(cfg, CFG_CSECTION, CFG_SHOW, true) ? mode_viewer
-                                                                  : mode_off;
-    ctx.info.timeout = config_get_num(cfg, CFG_CSECTION, CFG_ITIMEOUT, 0, 1024,
-                                      CFG_ITIMEOUT_DEF);
+    ctx.mode =
+        config_get_bool(cfg, CFG_INFO, CFG_INFO_SHOW) ? mode_viewer : mode_off;
+    ctx.info.timeout =
+        config_get_num(cfg, CFG_INFO, CFG_INFO_ITIMEOUT, 0, 1024);
     timeout_init(&ctx.info);
 
-    ctx.status.timeout = config_get_num(cfg, CFG_CSECTION, CFG_STIMEOUT, 0,
-                                        1024, CFG_STIMEOUT_DEF);
+    ctx.status.timeout =
+        config_get_num(cfg, CFG_INFO, CFG_INFO_STIMEOUT, 0, 1024);
     timeout_init(&ctx.status);
 
     font_render("File name:", &ctx.fields[info_file_name].key);
+    font_render("Directory:", &ctx.fields[info_file_dir].key);
     font_render("File path:", &ctx.fields[info_file_path].key);
     font_render("File size:", &ctx.fields[info_file_size].key);
     font_render("Image format:", &ctx.fields[info_image_format].key);
@@ -567,6 +539,7 @@ void info_reset(const struct image* image)
         (float)image->file_size / (image->file_size >= mib ? mib : 1024);
 
     font_render(image->name, &ctx.fields[info_file_name].value);
+    font_render(image->parent_dir, &ctx.fields[info_file_dir].value);
     font_render(image->source, &ctx.fields[info_file_path].value);
     font_render(image->format, &ctx.fields[info_image_format].value);
 
