@@ -10,6 +10,7 @@
 /** Available info fields. */
 enum info_field {
     info_file_name,
+    info_file_dir,
     info_file_path,
     info_file_size,
     info_image_format,
@@ -25,7 +26,7 @@ enum info_field {
  * Initialize global info context.
  * @param cfg config instance
  */
-void info_init(struct config* cfg);
+void info_init(const struct config* cfg);
 
 /**
  * Destroy global info context.
