@@ -26,7 +26,7 @@
     enum loader_status LOADER_FUNCTION(name)(struct image * ctx, \
                                              const uint8_t* data, size_t size)
 
-const char* supported_formats = "bmp, pnm, farbfeld, tga"
+const char* supported_formats = "bmp, pnm, farbfeld, tga, dicom"
 #ifdef HAVE_LIBJPEG
                                 ", jpeg"
 #endif
@@ -60,8 +60,8 @@ const char* supported_formats = "bmp, pnm, farbfeld, tga"
 #ifdef HAVE_LIBTIFF
                                 ", tiff"
 #endif
-#ifdef HAVE_LIBDICOM
-                                ", dicom"
+#ifdef HAVE_LIBSIXEL
+                                ", sixel"
 #endif
     ;
 
@@ -99,6 +99,9 @@ LOADER_DECLARE(svg);
 #ifdef HAVE_LIBTIFF
 LOADER_DECLARE(tiff);
 #endif
+#ifdef HAVE_LIBSIXEL
+LOADER_DECLARE(sixel);
+#endif
 #ifdef HAVE_LIBWEBP
 LOADER_DECLARE(webp);
 #endif
@@ -138,6 +141,9 @@ static const image_decoder decoders[] = {
 #endif
 #ifdef HAVE_LIBTIFF
     &LOADER_FUNCTION(tiff),
+#endif
+#ifdef HAVE_LIBSIXEL
+    &LOADER_FUNCTION(sixel),
 #endif
     &LOADER_FUNCTION(qoi),
     &LOADER_FUNCTION(farbfeld),
@@ -202,17 +208,16 @@ static enum loader_status image_from_file(struct image* img, const char* file)
     struct stat st;
     int fd;
 
-    // open file and get its size
+    // check file type
+    if (stat(file, &st) == -1 || !S_ISREG(st.st_mode)) {
+        return ldr_ioerror;
+    }
+
+    // open file and map it to memory
     fd = open(file, O_RDONLY);
     if (fd == -1) {
         return ldr_ioerror;
     }
-    if (fstat(fd, &st) == -1) {
-        close(fd);
-        return ldr_ioerror;
-    }
-
-    // map file to memory
     data = mmap(NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
     if (data == MAP_FAILED) {
         close(fd);
@@ -318,17 +323,10 @@ enum loader_status loader_from_source(const char* source, struct image** image)
     struct image* img;
 
     // create image instance
-    img = image_create();
+    img = image_alloc();
     if (!img) {
         return ldr_ioerror;
     }
-    img->source = str_dup(source, NULL);
-    img->name = strrchr(img->source, '/');
-    if (!img->name || strcmp(img->name, "/") == 0) {
-        img->name = img->source;
-    } else {
-        ++img->name; // skip slash
-    }
 
     // decode image
     if (strcmp(source, LDRSRC_STDIN) == 0) {
@@ -340,6 +338,7 @@ enum loader_status loader_from_source(const char* source, struct image** image)
     }
 
     if (status == ldr_success) {
+        image_set_source(img, source);
         *image = img;
     } else {
         image_free(img);
