@@ -12,6 +12,7 @@
 #include "pixmap_scale.h"
 #include "ui.h"
 
+#include <math.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/timerfd.h>
@@ -77,7 +78,9 @@ static const char* position_names[] = {
 
 /** Viewer context. */
 struct viewer {
-    ssize_t img_x, img_y;        ///< Top left corner of the image
+    ssize_t img_x, img_y; ///< Top left corner of the image
+    ssize_t img_w, img_h; ///< Image width and height
+
     size_t frame;                ///< Index of the current frame
     argb_t image_bkg;            ///< Image background mode/color
     argb_t window_bkg;           ///< Window background mode/color
@@ -87,7 +90,7 @@ struct viewer {
     enum fixed_scale scale_init; ///< Initial scale
     bool keep_zoom;              ///< Keep absolute zoom across images
     enum position position;      ///< Initial position
-    float scale;                 ///< Current scale factor of the image
+    double scale;                ///< Current scale factor of the image
 
     bool animation_enable; ///< Animation enable/disable
     int animation_fd;      ///< Animation timer
@@ -383,6 +386,16 @@ static void scale_global(const char* params)
     app_redraw();
 }
 
+/**
+ * Toggle zoom keeping mode.
+ */
+static void toggle_keep_zoom(void)
+{
+    ctx.keep_zoom = !ctx.keep_zoom;
+    info_update(info_status, "Keep zoom %s", ctx.keep_zoom ? "ON" : "OFF");
+    app_redraw();
+}
+
 /**
  * Start/stop animation if image supports it.
  * @param enable state to set
@@ -430,16 +443,27 @@ static void reset_state(void)
     const size_t total_img = image_list_size();
 
     ctx.frame = 0;
+
     if (!ctx.keep_zoom || ctx.scale == 0) {
         scale_image(ctx.scale_init);
+        set_position();
+    } else {
+        const ssize_t diff_w = ctx.img_w - img->frames[0].pm.width;
+        const ssize_t diff_h = ctx.img_h - img->frames[0].pm.height;
+        ctx.img_x += floor(ctx.scale * diff_w) / 2.0;
+        ctx.img_y += floor(ctx.scale * diff_h) / 2.0;
+        fixup_position(true);
     }
-    set_position();
+
+    ctx.img_w = img->frames[0].pm.width;
+    ctx.img_h = img->frames[0].pm.height;
 
     ui_set_title(img->name);
     animation_ctl(true);
     slideshow_ctl(ctx.slideshow_enable);
 
     info_reset(img);
+    info_update(info_scale, "%.0f%%", ctx.scale * 100);
     if (total_img) {
         info_update(info_index, "%zu of %zu", img->index + 1, total_img);
     }
@@ -578,8 +602,8 @@ static void draw_image(struct pixmap* wnd)
     // clear image background
     if (img->alpha) {
         if (ctx.image_bkg == GRID_BKGID) {
-            pixmap_grid(wnd, ctx.img_x, ctx.img_y, width, height,
-                        GRID_STEP, GRID_COLOR1, GRID_COLOR2);
+            pixmap_grid(wnd, ctx.img_x, ctx.img_y, width, height, GRID_STEP,
+                        GRID_COLOR1, GRID_COLOR2);
         } else {
             pixmap_fill(wnd, ctx.img_x, ctx.img_y, width, height,
                         ctx.image_bkg);
@@ -694,6 +718,9 @@ static void apply_action(const struct action* action)
         case action_scale:
             scale_global(action->params);
             break;
+        case action_keep_zoom:
+            toggle_keep_zoom();
+            break;
         case action_rotate_left:
             rotate_image(false);
             break;
